/**
 * AMPSSubscribeForReplay
 *
 * This sample subscribes to a topic in AMPS that maintains a transaction
 * log, and requests replay from the transaction log.
 *
 * The program flow is simple:
 *
 * - Connect to AMPS
 * - Logon
 * - Request replay from the "messages-history" topic from the last
 *   message received.
 *
 * To understand this program, start with the subscribe method of the
 * bookmark_subscriber.
 *
 * This file is a part of the AMPS Evaluation Kit.
 *************************************************************************
 **
 ** Copyright (c) 2010-2017 60East Technologies Inc., All Rights Reserved.
 **
 ** This computer software is owned by 60East Technologies Inc. and is
 ** protected by U.S. copyright laws and other laws and by international
 ** treaties.  This computer software is furnished by 60East Technologies
 ** Inc. pursuant to a written license agreement and may be used, copied,
 ** transmitted, and stored only in accordance with the terms of such
 ** license agreement and with the inclusion of the above copyright notice.
 ** This computer software or any other copies thereof may not be provided
 ** or otherwise made available to any other person.
 **
 ** U.S. Government Restricted Rights.  This computer software: (a) was
 ** developed at private expense and is in all respects the proprietary
 ** information of 60East Technologies Inc.; (b) was not developed with
 ** government funds; (c) is a trade secret of 60East Technologies Inc.
 ** for all purposes of the Freedom of Information Act; and (d) is a
 ** commercial item and thus, pursuant to Section 12.212 of the Federal
 ** Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
 ** Government's use, duplication or disclosure of the computer software
 ** is subject to the restrictions set forth by 60East Technologies Inc..
 **
 ************************************************************************
 **/

const amps = require('amps')


const uri = 'ws://127.0.0.1:9127/amps/json'
const client = new amps.Client('exampleSubscriberWithReplay')


let messageCount = 0

// Handler for received messages.
//
// Prints the received message to the console and
// discards the message from the bookmark store.
function onMessagePrinter(message) {
    messageCount++
    console.log(message.data)

    // Discard the message. This tells the bookmark store that the
    // message has been processed, and does not need to be requested
    // again.
    client.bookmarkStore().discard(message)

    // After 10 messages, exit the program
    if (messageCount % 20 === 0 && messageCount < 1000) {
        // create a disconnection to simulate connection issues
        client.connection.close()
    }
    // replayed all messages, disconnecting
    else if (messageCount === 1000) {
        client.disconnect()
    }
}


// onErrorHandler
//
// Receives any errors thrown by the message processor. For demonstration
// purposes, the handler prints the error and continues.
function onErrorHander(err) {
    console.error('err: ', err)
}


// Main section of the program.
async function main() {
    try {
        // Create a server chooser. For demonstration purposes, only include
        // the URI to the sample AMPS instance.
        const chooser = new amps.DefaultServerChooser()
        chooser.add(uri)
        client.serverChooser(chooser)

        // Assign the bookmark store and subscription manager.
        client.subscriptionManager(new amps.DefaultSubscriptionManager())
        client.bookmarkStore(new amps.MemoryBookmarkStore())

        // Set the error handler.
        client.errorHandler(onErrorHander)

        // The client is now prepared for use. Connect and logon.
        await client.connect(uri)

        // Enter the subscription. This statement requests all
        // messages that have not been previously processed on
        // the "messages-history" topic.
        await client.subscribe(
            onMessagePrinter,
            'messages-history',
            null,
            {bookmark: amps.Client.Bookmarks.MOST_RECENT}
        )
    }
    catch (err) {
        console.error('err: ', err)
    }
}
main()
