/**
 * AMPSUpdateForOOF
 *
 * This sample publishes messages to a SOW topic, then updates
 * the message in a way that will cause AMPS to deliver OOF
 * messages to the sample program that requests OOF notifications.
 *
 * The program flow is:
 *
 * - Connect and logon to AMPS
 * - Publish a message with expiration set
 * - Publish a message to be deleted later
 * - Publish a set of messages
 * - Publish updates to some of those messages
 * - Delete the message published to be deleted
 *
 * During this process, the sample program that receives OOF notifications
 * will receive OOF messages for updates, for an expiration, and for a
 * deleted message.
 *
 * This sample doesn't include error handling, high availability, or
 * connection retry logic.
 *
 * This file is a part of the AMPS Evaluation Kit.
 *************************************************************************
 **
 ** Copyright (c) 2010-2017 60East Technologies Inc., All Rights Reserved.
 **
 ** This computer software is owned by 60East Technologies Inc. and is
 ** protected by U.S. copyright laws and other laws and by international
 ** treaties.  This computer software is furnished by 60East Technologies
 ** Inc. pursuant to a written license agreement and may be used, copied,
 ** transmitted, and stored only in accordance with the terms of such
 ** license agreement and with the inclusion of the above copyright notice.
 ** This computer software or any other copies thereof may not be provided
 ** or otherwise made available to any other person.
 **
 ** U.S. Government Restricted Rights.  This computer software: (a) was
 ** developed at private expense and is in all respects the proprietary
 ** information of 60East Technologies Inc.; (b) was not developed with
 ** government funds; (c) is a trade secret of 60East Technologies Inc.
 ** for all purposes of the Freedom of Information Act; and (d) is a
 ** commercial item and thus, pursuant to Section 12.212 of the Federal
 ** Acquisition Regulations (FAR) and DFAR Supplement Section 227.7202,
 ** Government's use, duplication or disclosure of the computer software
 ** is subject to the restrictions set forth by 60East Technologies Inc..
 **
 ************************************************************************
 **/

const amps = require('amps')

// deletedMessageAck
//
// This function receives the acknowledgement message that results from
// deleting a message from the SOW. For sample purposes, this program
// simply prints the contents of the message.
function deleteMessageAck(message) {
    console.log('Delete ack: ', message);
}


// main program
//
// Connect to AMPS, send a message with an expiration, and send a message that
// will be deleted. After that, send a set of messages and update the messages.
async function main() {
    const uri = 'ws://127.0.0.1:9127/amps/json'

    const client = new amps.Client('exampleSOWUpdater')

    try {
        await client.connect(uri)

        // Publish a message with the expiration set.
        const cmd = new amps.Command('publish').expiration(1).topic('messages-sow')
        cmd.data({messageNumber: 50000, message: 'Here and then gone...'})

        client.execute(cmd)

        // Publish a message to be deleted later on.
        client.publish(
            'messages-sow',
            {messageNumber: 500, message: 'I\'ve got a bad feeling about this...'}
        )

        // Publish two sets of messages, the first one to match
        // the subscriber filter, the next one to make messages no longer
        // match the subscriber filter.

        // The first set of messages is designed so that, if this is run
        // after the previous SOW example, the sample with OOF tracking
        // recieves an updated message.
        for (let i = 0; i < 10000; i += 1250) {
            client.publish('messages-sow', {
                messageNumber: i,
                message: 'Hello, you crazy world!',
                OptionalField: 'true'
            })
        }

        // The second set of messages will cause OOF messages for the sample
        // that uses OOF tracking.
        for (let i = 0; i < 10000; i += 1250) {
            client.publish('messages-sow', {
                messageNumber: i,
                message: 'Updated, world!',
                OptionalField: 'ignore_me'
            })
        }

        // Delete the message published earlier
        deleteMessageAck(await client.sowDelete('messages-sow', '/messageNumber = 500'))

        await client.disconnect()
    }
    catch (err) {
        console.error('err: ', err)
    }
}
main()
